import React, { PureComponent } from 'react';
import { Select, Input, Button, Icon } from 'antd';
import MultipleSelectTable from './MultipleSelectTable';
import SingleSelectTable from './SingleSelectTable';
import { getSelectorColumns } from './columnsMap';
import { addRowKey } from '../../utils/utils';
import styles from './Selector.less';

function getRowKeys(rows) {
  return rows.map(item => item.key);
}

export default class Selector extends PureComponent {
  constructor(props) {
    super(props);
    const rows = addRowKey(props.selectedRows || []);
    const rowKeys = getRowKeys(rows || []);
    this.state = {
      searchKey: 'name',
      searchValue: '',
      selectedRows: rows,
      selectedRowKeys: rowKeys,
    };
  }
  componentWillReceiveProps(nextProps) {
    this.setState({
      selectedRows: addRowKey(nextProps.selectedRows),
      selectedRowKeys: getRowKeys(addRowKey(nextProps.selectedRows) || []),
    });
  }
  /**
   * 选择搜索字段
   * @param value <code|name>
   */
  handleSelectChange = (value) => {
    this.setState({
      searchKey: value,
    });
  };
  /**
   * 搜索输入框内容改变回调
   * @param e
   */
  handleInputChange = (e) => {
    this.setState({
      searchValue: e.target.value,
    });
  };
  /**
   * 过滤内容
   * @param page
   * @param pageSize
   */
  handleFilterOperation = (page, pageSize) => {
    const { searchKey, searchValue } = this.state;
    this.props.onChange({
      pageSize,
      pageNo: page,
      [searchKey]: searchValue,
    });
  };
  /**
   * 点击`搜索`按钮回调
   */
  handleSearchOperation = () => {
    const { searchKey, searchValue } = this.state;
    this.props.onChange({
      [searchKey]: searchValue,
    });
  };
  /**
   * 清除搜索框内容回调
   */
  handleInputClearOperation = () => {
    this.setState({
      searchValue: '',
    });
    this.props.onChange({});
  };
  /**
   * 待选列表中的选中项放到已选列表中
   * @params rows
   */
  handleMultipleTransfer = (rows) => {
    const { selectedRows, selectedRowKeys } = this.state;
    const newSelectedRowKeys = [];
    const newSelectedRows = rows.filter((item) => {
      if (selectedRowKeys.indexOf(item.key) === -1) {
        newSelectedRowKeys.push(item.key);
        return true;
      }
      return false;
    });
    this.setState({
      selectedRows: [...selectedRows, ...newSelectedRows],
      selectedRowKeys: [...selectedRowKeys, ...newSelectedRowKeys],
    });
  };
  /**
   * 单选模式选中的项,更新到state
   * @param row
   */
  handleSingleTransfer = (row) => {
    this.setState({
      selectedRows: [row],
    });
  };
  /**
   * 点击`移除`按钮回调
   * @params removeRows
   */
  handleRemoveOperation = (removeRows) => {
    const { selectedRows } = this.state;
    const newSelectedRowKeys = [];
    const removeRowKeys = removeRows.map(item => item.key);
    const newSelectedRows = selectedRows.filter((item) => {
      if (removeRowKeys.indexOf(item.key) === -1) {
        newSelectedRowKeys.push(item.key);
        return true;
      }
      return false;
    });
    this.setState({
      selectedRows: newSelectedRows,
      selectedRowKeys: newSelectedRowKeys,
    });
  };
  /**
   * 点击`完成`按钮回调
   */
  handleFinishOperation = () => {
    const { selectedRows } = this.state;
    this.props.onFinish(selectedRows);
  };
  /**
   * 点击`取消`按钮回调
   */
  handleCancelOperation = () => {
    this.props.onCancel();
  };

  render() {
    const {
      pageNo, pageSize, totalSize, loading, list, multiple, selectorName, fixedName,
    } = this.props;
    const { selectedRows, searchKey, searchValue } = this.state;

    const pagination = {
      pageSize,
      current: pageNo,
      total: totalSize,
    };

    const renderMultipleListScope = () => {
      return (
        <div className={styles.content}>
          <div className={styles.left}>
            <MultipleSelectTable
              tableType="waiting"
              dataSource={addRowKey(list)}
              loading={loading}
              pagination={pagination}
              onChange={this.handleFilterOperation}
              onTransfer={this.handleMultipleTransfer}
              columns={getSelectorColumns(selectorName)}
            />
          </div>
          <div className={styles.center}>
            <Icon className={styles.icon} type="forward" />
          </div>
          <div className={styles.right}>
            <MultipleSelectTable
              tableType="selected"
              loading={false}
              pagination={false}
              dataSource={selectedRows}
              onRemove={this.handleRemoveOperation}
              columns={getSelectorColumns(fixedName || selectorName)}
            />
          </div>
        </div>
      );
    };
    const renderSingleListScope = () => {
      return (
        <div className={styles.content}>
          <SingleSelectTable
            loading={loading}
            dataSource={addRowKey(list)}
            pagination={pagination}
            columns={getSelectorColumns(selectorName)}
            onChange={this.handleFilterOperation}
            onSingleTransfer={this.handleSingleTransfer}
          />
        </div>
      );
    };

    return (
      <div className={styles.wrapper}>
        <div className={styles.header}>
          {/* 这里写死了，搜索的字段一般就name和code */}
          <div className={styles.select}>
            <Select value={searchKey} onChange={this.handleSelectChange}>
              <Select.Option key="code" value="code">编号</Select.Option>
              <Select.Option key="name" value="name">名称</Select.Option>
            </Select>
          </div>
          <div className={styles.input}>
            <Input
              suffix={
                searchValue ? (
                  <Icon
                    type="close-circle"
                    onClick={this.handleInputClearOperation}
                  />
                ) : null
              }
              value={searchValue}
              onChange={this.handleInputChange}
              onPressEnter={this.handleSearchOperation}
              placeholder="请输入"
            />
          </div>
          <div className={styles.button}>
            <Button
              type="primary"
              icon="search"
              onClick={this.handleSearchOperation}
            >搜索
            </Button>
          </div>
        </div>
        {multiple ? renderMultipleListScope() : renderSingleListScope()}
        <div className={styles.footer}>
          <Button
            onClick={this.handleCancelOperation}
          >取消
          </Button>
          <Button
            type="primary"
            style={{ marginLeft: 20 }}
            onClick={this.handleFinishOperation}
          >完成
          </Button>
        </div>
      </div>
    );
  }
}
