import React, { Component } from 'react';
import moment from 'moment';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Card, Modal, Button, message } from 'antd';
import Ellipsis from '../../../components/Ellipsis';
import { StandardTableList } from '../../../components/AXList';
import {
  addRowKey,
  renderStatus,
  renderAvatar,
  renderGender,
  renderCategory,
} from '../../../utils/utils';
import { Hotax } from '../../../utils/config';

const Message = message;

@connect(({ loading, cmsUser }) => ({
  cmsUser,
  loading: loading.models.cmsUser,
}))
export default class CmsUserListPage extends Component {
  constructor(props) {
    super(props);
    const { state } = props.location;
    this.state = {
      UIParams: (state || {}).UIParams, // 组件的状态参数
      Queryers: (state || {}).Queryers, // 查询的条件参数
    };
  }
  componentDidMount() {
    this.props.dispatch({
      type: 'cmsUser/fetchCmsUserList',
      payload: { ...this.state.Queryers },
    });
  }
  handleCreateOperation = () => {
    this.props.dispatch(routerRedux.push({
      pathname: '/system/cms-user/create',
      state: this.state,
    }));
  }
  handleDisableOperation = (item) => {
    Modal.confirm({
      okText: '禁用',
      cancelText: '取消',
      title: '你确定要禁用该系统账户吗?',
      content: '禁用该账户后将不能使用该账户登录管理系统',
      onOk: () => {
        this.props.dispatch({
          type: 'cmsUser/deleteCmsUserItem',
          payload: { id: item.id },
          states: this.state,
        });
      },
    });
  }
  handleRecoverOperation = (item) => {
    Modal.confirm({
      okText: '确定',
      cancelText: '取消',
      title: '你确定要解除对该系统账号的禁用状态吗?',
      onOk: () => {
        this.props.dispatch({
          type: 'cmsUser/recoverCmsUserItem',
          payload: {
            id: item.id,
            status: Hotax.STATUS_NORMAL,
          },
          states: this.state,
        });
      },
    });
  }
  handleEditOperation = (item) => {
    this.props.dispatch(routerRedux.push({
      pathname: `/system/cms-user/edit/${item.id}`,
      state: {
        currentItem: item,
        ...this.state,
      },
    }));
  }
  handleFilterOperation = (params, states) => {
    this.props.dispatch({
      type: 'cmsUser/fetchCmsUserList',
      payload: params,
    });
    this.setState({
      UIParams: states,
      Queryers: params,
    });
  }
  handleBatchOperation = () => {
    Message.info('暂不支持批量操作!');
  }

  render() {
    const { loading, cmsUser } = this.props;
    const { list, totalSize, pageSize, pageNo } = cmsUser;

    const renderOperation = (item) => {
      return (
        <div>
          <Button
            size="small"
            className="editBtn"
            onClick={() => this.handleEditOperation(item)}
          >编辑
          </Button>
          {
            item.status === Hotax.STATUS_NORMAL ? (
              <Button
                size="small"
                className="delBtn"
                onClick={() => this.handleDisableOperation(item)}
              >禁用
              </Button>
            ) : (
              <Button
                size="small"
                className="delBtn"
                onClick={() => this.handleRecoverOperation(item)}
              >解禁
              </Button>
            )
          }
        </div>
      );
    };

    const batchActions = [{
      key: 'delete',
      name: '批量禁用',
    }, {
      key: 'recovery',
      name: '批量解禁',
    }];
    const basicSearch = {
      keys: [{
        name: '用户名称',
        field: 'name',
      }],
    };
    const pagination = {
      pageNo,
      pageSize,
      totalSize,
    };
    const columns = [{
      title: '用户头像',
      key: 1,
      dataIndex: 'avatar',
      render: (text, record) => {
        if (record.nickName) {
          return renderAvatar(text, record.nickName);
        }
        return renderAvatar(text, record.name);
      },
      width: '10%',
    }, {
      title: '用户名称',
      key: 2,
      dataIndex: 'name',
      width: '10%',
      render: text => (
        <Ellipsis tooltip lines={1}>{text}</Ellipsis>
      ),
    }, {
      title: '用户昵称',
      key: 3,
      dataIndex: 'nickName',
      width: '10%',
    }, {
      title: '用户性别',
      key: 4,
      dataIndex: 'gender',
      render: text => renderGender(text),
      width: '10%',
    }, {
      title: '所属商户',
      key: 5,
      dataIndex: 'merchantName',
      width: '11%',
    }, {
      title: '商户类型',
      key: 6,
      dataIndex: 'domain',
      render: text => renderCategory(text),
      width: '10%',
    }, {
      title: '账号状态',
      key: 7,
      dataIndex: 'status',
      render: text => renderStatus(text, '已禁用'),
      width: '11%',
    }, {
      title: '更新时间',
      key: 8,
      dataIndex: 'gmtModified',
      render: text => moment(text).format('YYYY-MM-DD HH:mm:ss'),
      width: '16%',
    }, {
      title: '操作',
      key: 9,
      dataIndex: 'operation',
      render: (_, record) => renderOperation(record),
      width: '12%',
      align: 'right',
    }];
    return (
      <Card>
        <StandardTableList
          columns={columns}
          loading={loading}
          dataSource={addRowKey(list)}
          header={{
            basicSearch,
            onFilterClick: this.handleFilterOperation,
            onCreateClick: this.handleCreateOperation,
          }}
          footer={{
            pagination,
            batchActions,
            onBatchClick: this.handleBatchOperation,
          }}
          keepUIState={{ ...this.state.UIParams }}
        />
      </Card>
    );
  }
}
